@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# vim: set et ts=4 sts=4
+# vim:et sts=4 sw=4
 #
 # ibus-table - The Tables engine for IBus
 #
@@ -55,8 +55,11 @@ class KeyEvent:
 
 class editor(object):
     '''Hold user inputs chars and preedit string'''
-    def __init__ (self,phrase_table_index,valid_input_chars, max_key_length, database, parser = tabdict.parse, deparser = tabdict.deparse, max_length = 64):
+    def __init__ (self, config, phrase_table_index,valid_input_chars, max_key_length, database, parser = tabdict.parse, deparser = tabdict.deparse, max_length = 64):
         self.db = database
+        self._config = config
+        self._name = self.db.get_ime_property('name')
+        self._config_section = "engine/Table/%s" % self._name
         self._pt = phrase_table_index
         self._parser = parser
         self._deparser = deparser
@@ -89,15 +92,18 @@ class editor(object):
         # self._caret: caret position in lookup_table
         self._caret = 0
         # self._onechar: whether we only select single character
-        self._onechar = False
+        self._onechar = self._config.get_value (self._config_section, "OneChar", False)
         # self._chinese_mode: the candidate filter mode,
         #   0 is simplify Chinese
         #   1 is traditional Chinese
         #   2 is Big charset mode, but simplify Chinese first
         #   3 is Big charset mode, but traditional Chinese first
         #   4 is Big charset mode.
-        # we use LC_CTYPE to determine which one to use
-        self._chinese_mode = self.get_chinese_mode()
+        # we use LC_CTYPE or LANG to determine which one to use
+        self._chinese_mode = self._config.get_value (
+                self._config_section,
+                "ChineseMode",
+                self.get_chinese_mode())
 
     def get_chinese_mode (self):
         '''Use LC_CTYPE in your box to determine the _chinese_mode'''
@@ -126,6 +132,10 @@ class editor(object):
     def change_chinese_mode (self):
         if self._chinese_mode != -1:
             self._chinese_mode = (self._chinese_mode +1 ) % 5
+        self._config.set_value (
+                self._config_section,
+                "ChineseMode",
+                self._chinese_mode )
 
     def clear (self):
         '''Remove data holded'''
@@ -481,109 +491,119 @@ class editor(object):
 
     def update_candidates (self):
         '''Update lookuptable'''
-        if (self._chars[0] == self._chars[2] and self._candidates[0]) \
-                or self._chars[1]:
-            # if no change in valid input char or we have invalid input,
-            # we do not do sql enquery
-            pass
+        # first check whether the IME have defined start_chars
+        if self.db.startchars and ( len(self._chars[0]) == 1 )\
+                and ( len(self._chars[1]) == 0 ) \
+                and ( self._chars[0][0] not in self.db.startchars):
+            self._u_chars.append ( self._chars[0][0] )
+            self._strings.insert ( self._cursor[0], self._chars[0][0] )
+            self._cursor [0] += 1
+            self.clear_input()
         else:
-            # check whether last time we have only one candidate
-            only_one_last = self.one_candidate()
-            # do enquiry
-            self._lookup_table.clean ()
-            self._lookup_table.show_cursor (False)
-            if self._tabkey_list:
-                # here we need to consider two parts, table and pinyin
-                # first table
-                if not self._py_mode:
-                    if self.db._is_chinese :
-                        bm_index = self._pt.index('category')
-                        if self._chinese_mode == 0:
-                            # simplify Chinese mode
-                            self._candidates[0] = self.db.select_words(\
-                                    self._tabkey_list, self._onechar, 1 )
-                        elif self._chinese_mode == 1:
-                            # traditional Chinese mode
-                            self._candidates[0] = self.db.select_words(\
-                                    self._tabkey_list, self._onechar, 2 )
-                        else:
-                            self._candidates[0] = self.db.select_words(\
-                                    self._tabkey_list, self._onechar )
-                    else:
-                        self._candidates[0] = self.db.select_words( self._tabkey_list, self._onechar )
-                else:
-                    self._candidates[0] = self.db.select_zi( self._tabkey_list )
-                self._chars[2] = self._chars[0][:]
-                    
+            if (self._chars[0] == self._chars[2] and self._candidates[0]) \
+                    or self._chars[1]:
+                # if no change in valid input char or we have invalid input,
+                # we do not do sql enquery
+                pass
             else:
-                self._candidates[0] =[]
-            if self._candidates[0]:
-                self._candidates[0] = self.filter_candidates (self._candidates[0])
-            if self._candidates[0]:
-                map ( self.ap_candidate,self._candidates[0] )
-            else:
-                if self._chars[0]:
-                    ## old manner:
-                    #if self._candidates[1]:
-                    #    #print self._candidates[1]
-                    #    self._candidates[0] = self._candidates[1]
-                    #    self._candidates[1] = []
-                    #    last_input = self.pop_input ()
-                    #    self.auto_commit_to_preedit ()
-                    #    res = self.add_input( last_input )
-                    #    print res
-                    #    return res
-                    #else:
-                    #    self.pop_input ()
-                    #    self._lookup_table.clean()
-                    #    self._lookup_table.show_cursor (False)
-                    #    return False
-                    ###################
-                    ## new manner, we add new char to invalid input
-                    ## chars
-                    if not self._chars[1]:
-                        # we don't have invalid input chars
-                        # here we need to check the last input
-                        # is a punctuation or not, if is a punct,
-                        # then we use old maner to summit the former valid
-                        # candidate
-                        if ascii.ispunct (self._chars[0][-1].encode('ascii')) \
-                                or len (self._chars[0][:-1]) \
-                                in self.db.pkeylens \
-                                or only_one_last:
-                            # because we use [!@#$%] to denote [12345]
-                            # in py_mode, so we need to distinguish them
-                            ## old manner:
-                            if self._py_mode:
-                                if self._chars[0][-1] in "!@#$%":
-                                    self._chars[0].pop() 
-                                    self._tabkey_list.pop()
-                                    return True
-
-                            if self._candidates[1]:
-                                self._candidates[0] = self._candidates[1]
-                                self._candidates[1] = []
-                                last_input = self.pop_input ()
-                                self.auto_commit_to_preedit ()
-                                res = self.add_input( last_input )
-                                return res
+                # check whether last time we have only one candidate
+                only_one_last = self.one_candidate()
+                # do enquiry
+                self._lookup_table.clean ()
+                self._lookup_table.show_cursor (False)
+                if self._tabkey_list:
+                    # here we need to consider two parts, table and pinyin
+                    # first table
+                    if not self._py_mode:
+                        if self.db._is_chinese :
+                            bm_index = self._pt.index('category')
+                            if self._chinese_mode == 0:
+                                # simplify Chinese mode
+                                self._candidates[0] = self.db.select_words(\
+                                        self._tabkey_list, self._onechar, 1 )
+                            elif self._chinese_mode == 1:
+                                # traditional Chinese mode
+                                self._candidates[0] = self.db.select_words(\
+                                        self._tabkey_list, self._onechar, 2 )
                             else:
-                                self.pop_input ()
-                                self._lookup_table.clean()
-                                self._lookup_table.show_cursor (False)
-                                return False
-                        else:    
-                            # this is not a punct or not a valid phrase
-                            # last time
-                            self._chars[1].append( self._chars[0].pop() )
-                            self._tabkey_list.pop()
+                                self._candidates[0] = self.db.select_words(\
+                                        self._tabkey_list, self._onechar )
+                        else:
+                            self._candidates[0] = self.db.select_words( self._tabkey_list, self._onechar )
                     else:
-                        pass
-                    self._candidates[0] =[]
+                        self._candidates[0] = self.db.select_zi( self._tabkey_list )
+                    self._chars[2] = self._chars[0][:]
+                        
                 else:
-                    self._lookup_table.clean()
-                    self._lookup_table.show_cursor (False)
-            self._candidates[1] = self._candidates[0]
+                    self._candidates[0] =[]
+                if self._candidates[0]:
+                    self._candidates[0] = self.filter_candidates (self._candidates[0])
+                if self._candidates[0]:
+                    map ( self.ap_candidate,self._candidates[0] )
+                else:
+                    if self._chars[0]:
+                        ## old manner:
+                        #if self._candidates[1]:
+                        #    #print self._candidates[1]
+                        #    self._candidates[0] = self._candidates[1]
+                        #    self._candidates[1] = []
+                        #    last_input = self.pop_input ()
+                        #    self.auto_commit_to_preedit ()
+                        #    res = self.add_input( last_input )
+                        #    print res
+                        #    return res
+                        #else:
+                        #    self.pop_input ()
+                        #    self._lookup_table.clean()
+                        #    self._lookup_table.show_cursor (False)
+                        #    return False
+                        ###################
+                        ## new manner, we add new char to invalid input
+                        ## chars
+                        if not self._chars[1]:
+                            # we don't have invalid input chars
+                            # here we need to check the last input
+                            # is a punctuation or not, if is a punct,
+                            # then we use old maner to summit the former valid
+                            # candidate
+                            if ascii.ispunct (self._chars[0][-1].encode('ascii')) \
+                                    or len (self._chars[0][:-1]) \
+                                    in self.db.pkeylens \
+                                    or only_one_last:
+                                # because we use [!@#$%] to denote [12345]
+                                # in py_mode, so we need to distinguish them
+                                ## old manner:
+                                if self._py_mode:
+                                    if self._chars[0][-1] in "!@#$%":
+                                        self._chars[0].pop() 
+                                        self._tabkey_list.pop()
+                                        return True
+
+                                if self._candidates[1]:
+                                    self._candidates[0] = self._candidates[1]
+                                    self._candidates[1] = []
+                                    last_input = self.pop_input ()
+                                    self.auto_commit_to_preedit ()
+                                    res = self.add_input( last_input )
+                                    return res
+                                else:
+                                    self.pop_input ()
+                                    self._lookup_table.clean()
+                                    self._lookup_table.show_cursor (False)
+                                    return False
+                            else:    
+                                # this is not a punct or not a valid phrase
+                                # last time
+                                self._chars[1].append( self._chars[0].pop() )
+                                self._tabkey_list.pop()
+                        else:
+                            pass
+                        self._candidates[0] =[]
+                    else:
+                        self._lookup_table.clean()
+                        self._lookup_table.show_cursor (False)
+                self._candidates[1] = self._candidates[0]
+            
         return True    
 
     def commit_to_preedit (self):
@@ -593,10 +613,11 @@ class editor(object):
         else:
             _p_index = 8
         try:
-            self._strings.insert(self._cursor[0], self._candidates[0][ self.get_cursor_pos() ][_p_index])
-            self._cursor [0] += 1
-            if self._py_mode:
-                self._zi = self._candidates[0][ self.get_cursor_pos() ][_p_index]
+            if self._candidates[0]:
+                self._strings.insert(self._cursor[0], self._candidates[0][ self.get_cursor_pos() ][_p_index])
+                self._cursor [0] += 1
+                if self._py_mode:
+                    self._zi = self._candidates[0][ self.get_cursor_pos() ][_p_index]
             self.over_input ()
             self.update_candidates ()
         except:
@@ -814,6 +835,7 @@ class tabengine (ibus.EngineBase):
 
     def __init__ (self, bus, obj_path, db ):
         super(tabengine,self).__init__ (bus,obj_path)
+        self._bus = bus
         self._lookup_table = ibus.LookupTable (tabengine._page_size)
         # this is the backend sql db we need for our IME
         # we receive this db from IMEngineFactory
@@ -846,11 +868,26 @@ class tabengine (ibus.EngineBase):
             if _c in tabdict.tab_key_list:
                 self._valid_input_chars.append(_c)
         del self._chars
+
+        # check whether we can use '=' and '-' for page_down/up
+        self._page_down_keys = [keysyms.Page_Down, keysyms.KP_Page_Down]
+        self._page_up_keys = [keysyms.Page_Up, keysyms.KP_Page_Up]
+        if '=' not in self._valid_input_chars \
+                and '-' not in self._valid_input_chars:
+            self._page_down_keys.append (keysyms.equal)
+            self._page_up_keys.append (keysyms.minus)
+        
         self._pt = self.db.get_phrase_table_index ()
         self._ml = int(self.db.get_ime_property ('max_key_length'))
         
+        # name for config section
+        self._name = self.db.get_ime_property('name')
+        self._config_section = "engine/Table/%s" % self._name
+        
+        # config module
+        self._config = self._bus.get_config ()
         # Containers we used:
-        self._editor = editor(self._pt, self._valid_input_chars, self._ml, self.db)
+        self._editor = editor(self._config, self._pt, self._valid_input_chars, self._ml, self.db)
 
         # some other vals we used:
         # self._prev_key: hold the key event last time.
@@ -858,14 +895,23 @@ class tabengine (ibus.EngineBase):
         self._prev_char = None
         self._double_quotation_state = False
         self._single_quotation_state = False
-        # [ENmode,TABmode] we get TABmode properties from db
+
+        # [EnMode,TabMode] we get TabMode properties from db
         self._full_width_letter = [
-                False, 
-                self.db.get_ime_property('def_full_width_letter').lower() == u'true'
+                self._config.get_value (self._config_section,
+                    "EnDefFullWidthLetter",
+                    False),
+                self._config.get_value (self._config_section, 
+                    "TabDefFullWidthLetter", 
+                    self.db.get_ime_property('def_full_width_letter').lower() == u'true' )
                 ]
         self._full_width_punct = [
-                False,
-                self.db.get_ime_property('def_full_width_punct').lower() == u'true'
+                self._config.get_value (self._config_section,
+                    "EnDefFullWidthPunct",
+                    False),
+                self._config.get_value (self._config_section, 
+                    "TabDefFullWidthPunct", 
+                    self.db.get_ime_property('def_full_width_punct').lower() == u'true' )
                 ]
         # some properties we will involved, Property is taken from scim.
         #self._setup_property = Property ("setup", _("Setup"))
@@ -873,19 +919,21 @@ class tabengine (ibus.EngineBase):
             self._auto_commit = self.db.get_ime_property('auto_commit').lower() == u'true'
         except:
             self._auto_commit = False
+        self._auto_commit = self._config.get_value (self._config_section, "AutoCommit",
+                self._auto_commit)
         # the commit phrases length
         self._len_list = [0]
         # connect to SpeedMeter
-        try:
-            bus = dbus.SessionBus()
-            user = os.path.basename( os.path.expanduser('~') )
-            self._sm_bus = bus.get_object ("org.ibus.table.SpeedMeter.%s"\
-                    % user, "/org/ibus/table/SpeedMeter")
-            self._sm =  dbus.Interface(self._sm_bus,\
-                    "org.ibus.table.SpeedMeter") 
-        except:
-            self._sm = None
-        self._sm_on = False
+        #try:
+        #    bus = dbus.SessionBus()
+        #    user = os.path.basename( os.path.expanduser('~') )
+        #    self._sm_bus = bus.get_object ("org.ibus.table.SpeedMeter.%s"\
+        #            % user, "/org/ibus/table/SpeedMeter")
+        #    self._sm =  dbus.Interface(self._sm_bus,\
+        #            "org.ibus.table.SpeedMeter") 
+        #except:
+        #    self._sm = None
+        #self._sm_on = False
         self._on = False
         self.reset ()
 
@@ -1019,12 +1067,36 @@ class tabengine (ibus.EngineBase):
             self._editor.r_shift ()
         elif property == u'onechar':
             self._editor._onechar = not self._editor._onechar
+            self._config.set_value( self._config_section,
+                    "OneChar",
+                    self._editor._onechar)
+
         elif property == u'acommit':
             self._auto_commit = not self._auto_commit
+            self._config.set_value( self._config_section,
+                    "AutoCommit",
+                    self._auto_commit)
         elif property == u'letter':
             self._full_width_letter [self._mode] = not self._full_width_letter [self._mode]
+            if self._mode:
+                self._config.set_value( self._config_section,
+                        "TabDefFullWidthLetter",
+                        self._full_width_letter [self._mode])
+            else:
+                self._config.set_value( self._config_section,
+                        "EnDefFullWidthLetter",
+                        self._full_width_letter [self._mode])
+
         elif property == u'punct':
             self._full_width_punct [self._mode] = not self._full_width_punct [self._mode]
+            if self._mode:
+                self._config.set_value( self._config_section,
+                        "TabDefFullWidthPunct",
+                        self._full_width_punct [self._mode])
+            else:
+                self._config.set_value( self._config_section,
+                        "EnDefFullWidthPunct",
+                        self._full_width_punct [self._mode])
         elif property == u'cmode':
             self._editor.change_chinese_mode()
             self.reset()
@@ -1096,12 +1168,12 @@ class tabengine (ibus.EngineBase):
         self._update_preedit ()
         self._update_aux ()
 
-    def add_string_len(self, astring):
-        if self._sm_on:
-            try:
-                self._sm.Accumulate(len(astring))
-            except:
-                pass
+    #def add_string_len(self, astring):
+    #    if self._sm_on:
+    #        try:
+    #            self._sm.Accumulate(len(astring))
+    #        except:
+    #            pass
     
     def commit_string (self,string):
         self._editor.clear ()
@@ -1266,13 +1338,13 @@ class tabengine (ibus.EngineBase):
             return True
         
         # Match speedmeter shift
-        if self._match_hotkey (key, keysyms.apostrophe, modifier.CONTROL_MASK):
-            self._sm_on = not self._sm_on
-            if self._sm_on:
-                self._sm.Show ()
-            else:
-                self._sm.Hide ()
-            return True
+        #if self._match_hotkey (key, keysyms.apostrophe, modifier.CONTROL_MASK):
+        #    self._sm_on = not self._sm_on
+        #    if self._sm_on:
+        #        self._sm.Show ()
+        #    else:
+        #        self._sm.Hide ()
+        #    return True
         
         # Ignore key release event now :)
         if key.mask & modifier.RELEASE_MASK:
@@ -1370,7 +1442,7 @@ class tabengine (ibus.EngineBase):
             #return (KeyProcessResult,whethercommit,commitstring)
             if sp_res[0]:
                 self.commit_string (sp_res[1])
-                self.add_string_len(sp_res[1])
+                #self.add_string_len(sp_res[1])
                 self.db.check_phrase (sp_res[1], sp_res[2])
             else:
                 if sp_res[1] == u' ':
@@ -1396,7 +1468,7 @@ class tabengine (ibus.EngineBase):
                 #return (whethercommit,commitstring)
                 if sp_res[0]:
                     self.commit_string (sp_res[1])
-                    self.add_string_len(sp_res[1])
+                    #self.add_string_len(sp_res[1])
                     self.db.check_phrase (sp_res[1],sp_res[2])
             
             res = self._editor.add_input ( unichr(key.code) )
@@ -1409,7 +1481,7 @@ class tabengine (ibus.EngineBase):
                 #return (KeyProcessResult,whethercommit,commitstring)
                 if sp_res[0]:
                     self.commit_string (sp_res[1] + key_char)
-                    self.add_string_len(sp_res[1])
+                    #self.add_string_len(sp_res[1])
                     self.db.check_phrase (sp_res[1],sp_res[2])
                     return True
                 else:
@@ -1424,19 +1496,21 @@ class tabengine (ibus.EngineBase):
                     #return (whethercommit,commitstring)
                     if sp_res[0]:
                         self.commit_string (sp_res[1])
-                        self.add_string_len(sp_res[1])
+                        #self.add_string_len(sp_res[1])
                         self.db.check_phrase (sp_res[1], sp_res[2])
                         return True
 
             self._update_ui ()
             return True
         
-        elif key.code in (keysyms.equal, keysyms.Page_Down, keysyms.KP_Page_Down) and self._editor._candidates[0]:
+        elif key.code in self._page_down_keys \
+                and self._editor._candidates[0]:
             res = self._editor.page_down()
             self._update_lookup_table ()
             return res
 
-        elif key.code in (keysyms.minus, keysyms.Page_Up, keysyms.KP_Page_Up) and self._editor._candidates[0]:
+        elif key.code in self._page_up_keys \
+                and self._editor._candidates[0]:
             res = self._editor.page_up ()
             self._update_lookup_table ()
             return res
@@ -1448,7 +1522,7 @@ class tabengine (ibus.EngineBase):
                 o_py = self._editor._py_mode
                 commit_string = self._editor.get_preedit_strings ()
                 self.commit_string (commit_string)
-                self.add_string_len(commit_string)
+                #self.add_string_len(commit_string)
                 if o_py != self._editor._py_mode:
                     self._refresh_properties ()
                     self._update_ui ()
@@ -1476,34 +1550,35 @@ class tabengine (ibus.EngineBase):
             self.register_properties (self.properties)
             self._refresh_properties ()
             self._update_ui ()
-            try:
-                if self._sm_on:
-                    self._sm.Show ()
-                else:
-                    self._sm.Hide ()
-            except:
-                pass
+            #try:
+            #    if self._sm_on:
+            #        self._sm.Show ()
+            #    else:
+            #        self._sm.Hide ()
+            #except:
+            #    pass
     
     def focus_out (self):
-        try:
-            self._sm.Hide()
-        except:
-            pass
+        #try:
+        #    self._sm.Hide()
+        #except:
+        #    pass
+        pass
 
     def enable (self):
-        try:
-            self._sm.Reset()
-        except:
-            pass
+        #try:
+        #    self._sm.Reset()
+        #except:
+        #    pass
         self._on = True
         self.focus_in()
 
     def disable (self):
         self.reset()
-        try:
-            self._sm.Hide()
-        except:
-            pass
+        #try:
+        #    self._sm.Hide()
+        #except:
+        #    pass
         self._on = False
 
 
@@ -1518,5 +1593,16 @@ class tabengine (ibus.EngineBase):
             self._update_lookup_table ()
             return True
         return False
+
+    # for further implementation :)
+    @classmethod
+    def CONFIG_VALUE_CHANGED(cls, bus, section, name, value):
+        config = bus.get_config()
+        if section != self._config_section:
+            return
     
-    
+    @classmethod
+    def CONFIG_RELOADED(cls, bus):
+        config = bus.get_config()
+        if section != self._config_section:
+            return
