@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# vim: set et sts=4 sw=4
+# vim:et sts=4 sw=4
 #
 # ibus-table - The Tables engine for IBus
 #
@@ -32,7 +32,7 @@ import time
 import re
 
 patt_r = re.compile(r'c([ea])(\d):(.*)')
-patt_p = re.compile(r'p(-{0,1}\d)(\d)')
+patt_p = re.compile(r'p(-{0,1}\d)(-{0,1}\d)')
 
 # first make some number index we will used :)
 #(MLEN, CLEN, M0, M1, M2, M3, M4, PHRASE, FREQ, USER_FREQ) = range (0,10)
@@ -101,8 +101,14 @@ class tabsqlitedb:
                       #'no_check_chars':u'',
                       'description':'A IME under IBus Table',
                       'layout':'us',
-                      'rules':''}
+                      'rules':'',
                       #'rules':'ce2:p11+p12+p21+p22;ce3:p11+p21+p22+p31;ca4:p11+p21+p31+p41'}
+                      'least_commit_length':'0',
+                      'start_chars':''
+                      # we use this entry for those IME, which don't
+                      # have rules to build up phrase, but still need
+                      # auto commit to preedit
+                      }
             # inital the attribute in ime table, which should be updated from mabiao
             for _name in ime_keys:
                 sqlstr = 'INSERT INTO main.ime (attr,val) VALUES (?,?);'
@@ -142,8 +148,8 @@ class tabsqlitedb:
         
         self.rules = self.get_rules ()
         self.pkeylens = []
-        if self.rules:
-            self.pkeylens = self.phrase_keys_len ()
+        self.pkeylens = self.phrase_keys_len ()
+        self.startchars = self.get_start_chars ()
         
         #self._no_check_chars = self.get_no_check_chars()
         # for fast gouci
@@ -163,7 +169,7 @@ class tabsqlitedb:
                 desc = self.get_database_desc (user_db)
                 if desc == None :
                     self.init_user_db (user_db)
-                elif desc["version"] != "0.4":
+                elif desc["version"] != "0.5":
                     new_name = "%s.%d" %(user_db, os.getpid())
                     print >> stderr, "Can not support the user db. We will rename it to %s" % new_name
                     self.old_phrases = self.extra_user_phrases( user_db )
@@ -349,7 +355,7 @@ class tabsqlitedb:
                         if res.group(1) == 'a':
                             rules['above'] = int(res.group(2))
                         _cms = res.group(3).split('+')
-                        if len(_cms) > int(self.get_ime_property('max_key_length')):
+                        if len(_cms) > self._mlen:
                             print 'rule: "%s" over max key length' % rule
                             break
                         for _cm in _cms:
@@ -367,12 +373,31 @@ class tabsqlitedb:
 
     def phrase_keys_len (self):
         '''Return the phrase possible key length'''
-        max_len = self.rules["above"]
-        try:
-            return map ( lambda x: len(self.rules[x]), range(2, max_len+1) )[:]
-        except:
-            return None
+        if self.rules:
+            max_len = self.rules["above"]
+            try:
+                return map ( lambda x: len(self.rules[x]), range(2, max_len+1) )[:]
+            except:
+                return []
+        else:
+            # we try to findout whether we have least commit len
+            # in ime table
+            try:
+                least_commit_len = int(self.get_ime_property('least_commit_length'))
+            except:
+                least_commit_len = 0
+            if least_commit_len > 0:
+                return range (least_commit_len, self._mlen + 1)
+            else:
+                return []
     
+    def get_start_chars (self):
+        '''return possible start chars of IME'''
+        try:
+            return self.get_ime_property('start_chars')
+        except:
+            return ''
+
     def get_no_check_chars (self):
         '''Get the characters which engine should not change freq'''
         _chars= self.get_ime_property('no_check_chars')
@@ -595,18 +620,13 @@ class tabsqlitedb:
     
     def create_indexes(self, database, commit=True):
         sqlstr = '''
-            DROP INDEX IF EXISTS %(database)s.goucima_index_z;
             CREATE INDEX IF NOT EXISTS %(database)s.goucima_index_z ON goucima (zi);
-            DROP INDEX IF EXISTS %(database)s.pinyin_index_i;
             CREATE INDEX IF NOT EXISTS %(database)s.pinyin_index_i ON pinyin (p0,p1,p2,p3,p4,p5,plen ASC, freq DESC);
-            VACUUM; 
             ''' % { 'database':database }
 
         sqlstr_t = '''
-            DROP INDEX IF EXISTS %(database)s.phrases_index_p;
             CREATE INDEX IF NOT EXISTS %(database)s.phrases_index_p ON phrases
             (%(tabkeystr)s mlen ASC, freq DESC, id ASC);
-            DROP INDEX IF EXISTS %(database)s.phrases_index_i;
             CREATE INDEX IF NOT EXISTS %(database)s.phrases_index_i ON phrases (phrase, mlen ASC);
             ''' 
         tabkeystr = ''
@@ -776,7 +796,7 @@ class tabsqlitedb:
             sqlstring = 'CREATE TABLE IF NOT EXISTS user_db.desc (name PRIMARY KEY, value);'
             self.db.executescript (sqlstring)
             sqlstring = 'INSERT OR IGNORE INTO user_db.desc  VALUES (?, ?);'
-            self.db.execute (sqlstring, ('version', '0.4'))
+            self.db.execute (sqlstring, ('version', '0.5'))
             sqlstring = 'INSERT OR IGNORE INTO user_db.desc  VALUES (?, DATETIME("now", "localtime"));'
             self.db.execute (sqlstring, ("create-time", ))
             self.db.commit ()
